/*
 * Copyright (c) 2016, Freescale Semiconductor, Inc.
 * Copyright 2016-2017 NXP
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * o Redistributions of source code must retain the above copyright notice, this list
 *   of conditions and the following disclaimer.
 *
 * o Redistributions in binary form must reproduce the above copyright notice, this
 *   list of conditions and the following disclaimer in the documentation and/or
 *   other materials provided with the distribution.
 *
 * o Neither the name of the copyright holder nor the names of its
 *   contributors may be used to endorse or promote products derived from this
 *   software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#define TEST // testing mode, using LED gpio instead of ADC sampling

/*******************************************************************************
 * Includes
 ******************************************************************************/

#include "fsl_debug_console.h"
#include "board.h"
#include "fsl_adc.h"
#include "fsl_dma.h"
#include "fsl_ctimer.h"
#include "clock_config.h"
#include "pin_mux.h"

/*******************************************************************************
 * Definitions
 ******************************************************************************/
#define CTIMER CTIMER1                  /* Timer 1 */
#define CTIMER_MAT0_OUT kCTIMER_Match_1 /* Match output 1 */
//#define BUS_CLK_FREQ CLOCK_GetFreq(kCLOCK_ApbClk) - compilation error
#define BUFFER_SIZE 200
//#define INTERVAL BUS_CLK_FREQ / 1024 - compilation error

/*******************************************************************************
 * Prototypes
 ******************************************************************************/
/* Callback for match channel 1 - ADC sampler */
void sampler_callback(uint32_t flags);

/* Array of function pointers for callback for each channel */
ctimer_callback_t ctimer_callback_table[] = {
    NULL, sampler_callback, NULL, NULL, NULL, NULL, NULL, NULL};

//#ifndef TEST
static void ADC_Configuration(void);
static void ADC_ConvertDMACallback(dma_handle_t *handle, void *param, bool transferDone, uint32_t intmode);
//#endif

/*******************************************************************************
 * Definitions
 ******************************************************************************/
//#ifndef TEST
#define DEMO_ADC_BASE ADC                           /* Demo base address */
#define DEMO_ADC_CHANNEL 8                          /* Select channel */
#define DEMO_ADC_CFG_IDX 0                          /* Select configuration */
#define DEMO_ADC_TRIGGER kADC_TriggerSelectSoftware /* Software trigger */
//#define DEMO_ADC_IRQ_ID ADC_IRQn                    /* IRQ number */
//#define DEMO_ADC_IRQ_HANDLER_FUNC ADC_IRQHandler    /* Interrupt handler */
#define DEMO_ADC_DMA_CHANNEL 14
//#endif


/*******************************************************************************
* Variables
******************************************************************************/
/* Match Configuration for Channel 0 */
static ctimer_match_config_t samplerMatchConfig;

//#ifndef TEST
uint32_t g_AdcVinn;
float g_AdcBandgap;

volatile bool g_Stop = false;
bool g_Buzy = false;

uint32_t i;
uint32_t data[BUFFER_SIZE];

static uint32_t currentInterval = 100000; //INTERVAL; - compilation error
//#endif

uint32_t g_AdcConvResult;
dma_handle_t g_adcDmaHandle;

/*******************************************************************************
 * Code
 ******************************************************************************/

void sampler_callback(uint32_t flags)
{
#ifndef TEST
	if (g_Stop) {
		return;
	}

	if (g_Buzy) {
		return;
	}
#endif

	// toggle RED
	GPIO_TogglePinsOutput(BOARD_LED_GPIO, 1U << BOARD_LED_RED_GPIO_PIN);

#ifdef TEST
    DMA_StartTransfer(&g_adcDmaHandle);
	ADC_DoSoftwareTrigger(DEMO_ADC_BASE);

#else

    if (i < BUFFER_SIZE) {
        /* Software triger if buffer not filled */
        DMA_StartTransfer(&g_adcDmaHandle);
        ADC_DoSoftwareTrigger(DEMO_ADC_BASE);
    } else {
    	g_Buzy = true;

    	/* print buffer */
    	i = 0;

        //PRINTF("\r\n");

        for (uint32_t c = 0; c < BUFFER_SIZE; c++)
        {
            float fresult = ADC_ConversionResult2Mv(DEMO_ADC_BASE, DEMO_ADC_CHANNEL, DEMO_ADC_CFG_IDX, g_AdcBandgap,
                                              g_AdcVinn, data[c]);
            PRINTF("%f %d\r\n", fresult, currentInterval);
            //PRINTF("%f\r\n", fresult);
        }

    	/* update interval */
//        if (currentInterval > 100) {
//        	currentInterval -= 100;
//
//            samplerMatchConfig.matchValue = currentInterval;
//            CTIMER_SetupMatch(CTIMER, CTIMER_MAT0_OUT, &samplerMatchConfig);
//        } else {
       	g_Stop = true;
//        }

        g_Buzy = false;
    }
#endif
}

////#ifndef TEST
//void DEMO_ADC_IRQ_HANDLER_FUNC(void)
//{
//    if (ADC_GetStatusFlags(DEMO_ADC_BASE) & kADC_DataReadyFlag)
//    {
//    	volatile uint32_t g_AdcConvResult;
//        g_AdcConvResult = ADC_GetConversionResult(DEMO_ADC_BASE);
//#ifndef TEST
//		data[i++] = g_AdcConvResult;
//#else
//		// toggle GREEN
//		GPIO_TogglePinsOutput(BOARD_LED_GPIO, 1U << BOARD_LED_GREEN_GPIO_PIN);
//#endif
//    }
//}
////#endif

static void ADC_ConvertDMACallback(dma_handle_t *handle, void *param, bool transferDone, uint32_t intmode)
{
	// toggle GREEN
	GPIO_TogglePinsOutput(BOARD_LED_GPIO, 1U << BOARD_LED_GREEN_GPIO_PIN);
}


/*!
 * @brief Main function
 */
int main(void)
{
	dma_transfer_config_t transferConfig;

#ifdef TEST
    /* Enable GPIO clock */
    CLOCK_EnableClock(kCLOCK_Gpio);

    /* Init output LED GPIO. */
    GPIO_PinInit(BOARD_LED_GPIO, BOARD_LED_RED_GPIO_PIN, &(gpio_pin_config_t){kGPIO_DigitalOutput, 1U});
    GPIO_PinInit(BOARD_LED_GPIO, BOARD_LED_GREEN_GPIO_PIN, &(gpio_pin_config_t){kGPIO_DigitalOutput, 1U});

#endif

    /* Power on ADC */
    POWER_EnableADC(true);

    /* Init hardware*/
    BOARD_InitPins();
    BOARD_BootClockRUN();
    BOARD_InitDebugConsole();

    /* Configure DMA. */
    DMA_Init(DMA0);
    DMA_EnableChannel(DMA0, DEMO_ADC_DMA_CHANNEL);
    DMA_CreateHandle(&g_adcDmaHandle, DMA0, DEMO_ADC_DMA_CHANNEL);
    DMA_SetCallback(&g_adcDmaHandle, ADC_ConvertDMACallback, NULL);
    DMA_PrepareTransfer(&transferConfig, (void *)&DEMO_ADC_BASE->DATA, &g_AdcConvResult, sizeof(uint32_t), 4,
                        kDMA_PeripheralToMemory, NULL);
    DMA_SubmitTransfer(&g_adcDmaHandle, &transferConfig);

//#ifndef TEST

    /* Configure the converter and work mode. */
    ADC_Configuration();
    /* Enable ADC interrupt */
    ADC_EnableInterrupts(DEMO_ADC_BASE, (kADC_InterruptMaskEnable | kADC_InterruptDataReadyEnable));
 ////   NVIC_EnableIRQ(DEMO_ADC_IRQ_ID);
//#endif

    ctimer_config_t config;
    CTIMER_GetDefaultConfig(&config);
    CTIMER_Init(CTIMER, &config);

    samplerMatchConfig.enableCounterReset = true;
    samplerMatchConfig.enableCounterStop = false;
    samplerMatchConfig.matchValue = currentInterval;
    samplerMatchConfig.outControl = kCTIMER_Output_Toggle;
    samplerMatchConfig.outPinInitState = false;
    samplerMatchConfig.enableInterrupt = true;

    CTIMER_RegisterCallBack(CTIMER, &ctimer_callback_table[0], kCTIMER_MultipleCallback);
    CTIMER_SetupMatch(CTIMER, CTIMER_MAT0_OUT, &samplerMatchConfig);
    CTIMER_StartTimer(CTIMER);


}

//#ifndef TEST
static void ADC_Configuration(void)
{
    adc_config_t adcConfigStruct;
    adc_sd_config_t adcSdConfigStruct;

    /**
     * Initial ADC to default configuration.
     */
    ADC_GetDefaultConfig(&adcConfigStruct);
    adcConfigStruct.channelEnable = (1U << DEMO_ADC_CHANNEL);
    adcConfigStruct.channelConfig = (DEMO_ADC_CFG_IDX << DEMO_ADC_CHANNEL);
    adcConfigStruct.triggerSource = DEMO_ADC_TRIGGER;
    adcConfigStruct.convMode = kADC_ConvModeSingle;
    ADC_Init(DEMO_ADC_BASE, &adcConfigStruct);

    /* Initial ADC Sigma Delta(SD) configuration */
    ADC_GetSdDefaultConfig(&adcSdConfigStruct);
    ADC_SetSdConfig(DEMO_ADC_BASE, DEMO_ADC_CFG_IDX, &adcSdConfigStruct);

    /* Bandgap voltage */
    g_AdcBandgap = ADC_GetBandgapCalibrationResult(DEMO_ADC_BASE, DEMO_ADC_CFG_IDX);

    /* Calibration VINN value */
    g_AdcVinn = ADC_GetVinnCalibrationResult(DEMO_ADC_BASE, &adcConfigStruct);

    /* Enable ADC */
    ADC_Enable(DEMO_ADC_BASE, true);
}
//#endif
